'TSD Graphics Demo
'The Propeller Graphics Demo adapted for the 4.3" TFT Screen
'(c)2009 Andy Schenk

CON
  _clkmode = xtal1+pll16x
  _clkfreq = 80_000_000

  _stack = ($2200 + $2200 + 100) >> 2   'accomodate display memory and stack  
  x_tiles = 15
  y_tiles = 9

  bitmap_base  = $3C00
  display_base = $5E00

  lines = 5
  thickness = 2

DAT   'LCD connection and settings
sync          long  0           'Sync indicator variable
basepin       long  16          'CUSTOMIZE this Pin settings!!
DEpin         long  24
PCLKpin       long  25
DONpin        long  10
BLpin         long  11
Brightness    long  22          '1..31
parpass       long  0[4]        'reserved for parameter passing 

VAR

  long  colors[y_tiles]

  long  mousex, mousey, rndm   
  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

OBJ

  lcd   : "TSD_GraphicsDriver"
  gr    : "graphics"

pub Main
  'Start up Display
  lcd.start(@colors, display_base, @sync)

  GraphicsDemo


PUB GraphicsDemo|i,j,k,dx,dy, kk, pp, pq, rr, numx, numchr,GraphicsColor 
  'init colors
  colors[0]:= $0CFC0000+%%0210_0000
  colors[1]:= $0CFC0000+%%0210_0000  
  colors[2]:= $0CFC0000+%%0210_0000  
  colors[3]:= $0CFC0000+%%1210_0000  
  colors[4]:= $0CFC0000+%%1220_0000     
  colors[5]:= $0CFC0000+%%1220_0000  
  colors[6]:= $0CFC0000+%%1120_0000  
  colors[7]:= $0CFC0000+%%2120_0000  
  colors[8]:= $0CFC0000+%%2130_0000  

  'init bouncing lines
  i := 1001
  j := 123123
  k := 8776434
  repeat i from 0 to lines - 1
    x[i] := ?j // 64
    y[i] := k? // 48
    repeat until xs[i] := k? ~> 29
    repeat until ys[i] := ?j ~> 29

  'start and setup graphics
  gr.start
  gr.setup(x_tiles, y_tiles, 120, 48, bitmap_base)
  mousex~
  mousey~

  repeat

    'clear bitmap
    gr.clear


    'draw spinning triangles
    gr.colorwidth(3,0)
    repeat i from 1 to 8
      gr.vec(0, 0, (k & $7F) << 3 + i << 5, k << 6 + i << 8, @vecdef)

    'draw expanding mouse crosshairs
    gr.colorwidth(2,k>>2)
    gr.pix(mousex, mousey, k>>4 & $7, @pixdef)

    'from time to time, throw snowballs
    ifnot cnt & $3000_0000
      gr.width(pq & $F)
      gr.color(2)
      pp := (pq & $F)*(pq & $F) + 5
      pq++
      gr.arc(mousex, mousey, pp, pp>>1, -k * 200, $200, 8, 0)
    else
      pq~

    'draw expanding pixel halo
    gr.colorwidth(1,k)
    gr.arc(0,0,80,30,-k<<5,$2000/9,9,0)
    
    'step bouncing lines
    repeat i from 0 to lines - 1
      if ||~x[i] > 60
        -xs[i]
      if ||~y[i] > 40
        -ys[i]
      x[i] += xs[i]
      y[i] += ys[i]

    'draw bouncing lines
    gr.colorwidth(1,thickness)
    gr.plot(~x[0], ~y[0])
    repeat i from 1 to lines - 1
      gr.line(~x[i],~y[i])
    gr.line(~x[0], ~y[0])

    'draw spinning stars and revolving crosshairs and dogs
    gr.colorwidth(2,0)
    repeat i from 0 to 7
      gr.vecarc(80,50,30,30,-(i<<10+k<<6),$40,-(k<<7),@vecdef2)
      gr.pixarc(-80,-40,30,30,i<<10+k<<6,0,@pixdef2)
      gr.pixarc(-80,-40,20,20,-(i<<10+k<<6),0,@pixdef)

    'draw small box with text
    gr.colorwidth(1,14)
    gr.box(60,-38,60,16)  
    gr.textmode(1,1,6,5)
    gr.colorwidth(2,0)
    gr.text(90,-30,@pchip)
                 
    'draw incrementing digit
    if not ++numx & 7
      numchr++
    if numchr < "0" or numchr > "9"
      numchr := "0"
    gr.textmode(8,8,6,5)
    gr.colorwidth(1,8)
    gr.text(-90,50,@numchr)
 
    'copy bitmap to display
    gr.copy(display_base)

    'increment counter that makes everything change
    k++

    
DAT 'data for Graphics_Demo

vecdef                  word    $4000+$2000/3*0         'triangle
                        word    50
                        word    $8000+$2000/3*1+1
                        word    50
                        word    $8000+$2000/3*2-1
                        word    50
                        word    $8000+$2000/3*0
                        word    50
                        word    0

vecdef2                 word    $4000+$2000/12*0        'star
                        word    50
                        word    $8000+$2000/12*1
                        word    20
                        word    $8000+$2000/12*2
                        word    50
                        word    $8000+$2000/12*3
                        word    20
                        word    $8000+$2000/12*4
                        word    50
                        word    $8000+$2000/12*5
                        word    20
                        word    $8000+$2000/12*6
                        word    50
                        word    $8000+$2000/12*7
                        word    20
                        word    $8000+$2000/12*8
                        word    50
                        word    $8000+$2000/12*9
                        word    20
                        word    $8000+$2000/12*10
                        word    50
                        word    $8000+$2000/12*11
                        word    20
                        word    $8000+$2000/12*0
                        word    50
                        word    0

pixdef                  word                            'crosshair
                        byte    2,7,3,3
                        word    %%00333000,%%00000000
                        word    %%03020300,%%00000000
                        word    %%30020030,%%00000000
                        word    %%32222230,%%00000000
                        word    %%30020030,%%02000000
                        word    %%03020300,%%22200000
                        word    %%00333000,%%02000000

pixdef2                 word                            'dog
                        byte    1,4,0,3
                        word    %%20000022
                        word    %%02222222
                        word    %%02222200
                        word    %%02000200

pchip                   byte    "Propeller",0           'text